@@ -2,6 +2,7 @@ use std::borrow::Cow;
 use std::io::BufReader;
 use std::path::{Path, PathBuf};
 use std::str::FromStr;
+use std::sync::OnceLock;
 
 use fs_err as fs;
 use thiserror::Error;
@@ -9,14 +10,17 @@ use tracing::warn;
 use url::Url;
 
 use uv_cache_info::CacheInfo;
-use uv_distribution_filename::EggInfoFilename;
+use uv_distribution_filename::{EggInfoFilename, ExpandedTags};
 use uv_fs::Simplified;
+use uv_install_wheel::WheelFile;
 use uv_normalize::PackageName;
 use uv_pep440::Version;
 use uv_pypi_types::{DirectUrl, MetadataError};
 use uv_redacted::DisplaySafeUrl;
 
-use crate::{DistributionMetadata, InstalledMetadata, InstalledVersion, Name, VersionOrUrlRef};
+use crate::{
+    BuildInfo, DistributionMetadata, InstalledMetadata, InstalledVersion, Name, VersionOrUrlRef,
+};
 
 #[derive(Error, Debug)]
 pub enum InstalledDistError {
@@ -38,6 +42,12 @@ pub enum InstalledDistError {
     #[error(transparent)]
     PackageNameParse(#[from] uv_normalize::InvalidNameError),
 
+    #[error(transparent)]
+    WheelFileParse(#[from] uv_install_wheel::Error),
+
+    #[error(transparent)]
+    ExpandedTagParse(#[from] uv_distribution_filename::ExpandedTagError),
+
     #[error("Invalid .egg-link path: `{}`", _0.user_display())]
     InvalidEggLinkPath(PathBuf),
 
@@ -59,9 +69,42 @@ pub enum InstalledDistError {
     },
 }
 
+#[derive(Debug, Clone)]
+pub struct InstalledDist {
+    pub kind: InstalledDistKind,
+    // Cache data that must be read from the `.dist-info` directory. These are safe to cache as
+    // the `InstalledDist` is immutable after creation.
+    metadata_cache: OnceLock<uv_pypi_types::ResolutionMetadata>,
+    tags_cache: OnceLock<Option<ExpandedTags>>,
+}
+
+impl From<InstalledDistKind> for InstalledDist {
+    fn from(kind: InstalledDistKind) -> Self {
+        Self {
+            kind,
+            metadata_cache: OnceLock::new(),
+            tags_cache: OnceLock::new(),
+        }
+    }
+}
+
+impl std::hash::Hash for InstalledDist {
+    fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
+        self.kind.hash(state);
+    }
+}
+
+impl PartialEq for InstalledDist {
+    fn eq(&self, other: &Self) -> bool {
+        self.kind == other.kind
+    }
+}
+
+impl Eq for InstalledDist {}
+
 /// A built distribution (wheel) that is installed in a virtual environment.
 #[derive(Debug, Clone, Hash, PartialEq, Eq)]
-pub enum InstalledDist {
+pub enum InstalledDistKind {
     /// The distribution was derived from a registry, like `PyPI`.
     Registry(InstalledRegistryDist),
     /// The distribution was derived from an arbitrary URL.
@@ -80,6 +123,7 @@ pub struct InstalledRegistryDist {
     pub version: Version,
     pub path: Box<Path>,
     pub cache_info: Option<CacheInfo>,
+    pub build_info: Option<BuildInfo>,
 }
 
 #[derive(Debug, Clone, Hash, PartialEq, Eq)]
@@ -91,6 +135,7 @@ pub struct InstalledDirectUrlDist {
     pub editable: bool,
     pub path: Box<Path>,
     pub cache_info: Option<CacheInfo>,
+    pub build_info: Option<BuildInfo>,
 }
 
 #[derive(Debug, Clone, Hash, PartialEq, Eq)]
@@ -136,36 +181,46 @@ impl InstalledDist {
 
             let name = PackageName::from_str(name)?;
             let version = Version::from_str(version)?;
-            let cache_info = Self::cache_info(path)?;
+            let cache_info = Self::read_cache_info(path)?;
+            let build_info = Self::read_build_info(path)?;
 
-            return if let Some(direct_url) = Self::direct_url(path)? {
+            return if let Some(direct_url) = Self::read_direct_url(path)? {
                 match Url::try_from(&direct_url) {
-                    Ok(url) => Ok(Some(Self::Url(InstalledDirectUrlDist {
-                        name,
-                        version,
-                        editable: matches!(&direct_url, DirectUrl::LocalDirectory { dir_info, .. } if dir_info.editable == Some(true)),
-                        direct_url: Box::new(direct_url),
-                        url: DisplaySafeUrl::from(url),
-                        path: path.to_path_buf().into_boxed_path(),
-                        cache_info,
-                    }))),
-                    Err(err) => {
-                        warn!("Failed to parse direct URL: {err}");
-                        Ok(Some(Self::Registry(InstalledRegistryDist {
+                    Ok(url) => Ok(Some(Self::from(InstalledDistKind::Url(
+                        InstalledDirectUrlDist {
                             name,
                             version,
+                            editable: matches!(&direct_url, DirectUrl::LocalDirectory { dir_info, .. } if dir_info.editable == Some(true)),
+                            direct_url: Box::new(direct_url),
+                            url: DisplaySafeUrl::from(url),
                             path: path.to_path_buf().into_boxed_path(),
                             cache_info,
-                        })))
+                            build_info,
+                        },
+                    )))),
+                    Err(err) => {
+                        warn!("Failed to parse direct URL: {err}");
+                        Ok(Some(Self::from(InstalledDistKind::Registry(
+                            InstalledRegistryDist {
+                                name,
+                                version,
+                                path: path.to_path_buf().into_boxed_path(),
+                                cache_info,
+                                build_info,
+                            },
+                        ))))
                     }
                 }
             } else {
-                Ok(Some(Self::Registry(InstalledRegistryDist {
-                    name,
-                    version,
-                    path: path.to_path_buf().into_boxed_path(),
-                    cache_info,
-                })))
+                Ok(Some(Self::from(InstalledDistKind::Registry(
+                    InstalledRegistryDist {
+                        name,
+                        version,
+                        path: path.to_path_buf().into_boxed_path(),
+                        cache_info,
+                        build_info,
+                    },
+                ))))
             };
         }
 
@@ -189,19 +244,23 @@ impl InstalledDist {
 
             if let Some(version) = file_name.version {
                 if metadata.is_dir() {
-                    return Ok(Some(Self::EggInfoDirectory(InstalledEggInfoDirectory {
-                        name: file_name.name,
-                        version,
-                        path: path.to_path_buf().into_boxed_path(),
-                    })));
+                    return Ok(Some(Self::from(InstalledDistKind::EggInfoDirectory(
+                        InstalledEggInfoDirectory {
+                            name: file_name.name,
+                            version,
+                            path: path.to_path_buf().into_boxed_path(),
+                        },
+                    ))));
                 }
 
                 if metadata.is_file() {
-                    return Ok(Some(Self::EggInfoFile(InstalledEggInfoFile {
-                        name: file_name.name,
-                        version,
-                        path: path.to_path_buf().into_boxed_path(),
-                    })));
+                    return Ok(Some(Self::from(InstalledDistKind::EggInfoFile(
+                        InstalledEggInfoFile {
+                            name: file_name.name,
+                            version,
+                            path: path.to_path_buf().into_boxed_path(),
+                        },
+                    ))));
                 }
             }
 
@@ -209,22 +268,26 @@ impl InstalledDist {
                 let Some(egg_metadata) = read_metadata(&path.join("PKG-INFO")) else {
                     return Ok(None);
                 };
-                return Ok(Some(Self::EggInfoDirectory(InstalledEggInfoDirectory {
-                    name: file_name.name,
-                    version: Version::from_str(&egg_metadata.version)?,
-                    path: path.to_path_buf().into_boxed_path(),
-                })));
+                return Ok(Some(Self::from(InstalledDistKind::EggInfoDirectory(
+                    InstalledEggInfoDirectory {
+                        name: file_name.name,
+                        version: Version::from_str(&egg_metadata.version)?,
+                        path: path.to_path_buf().into_boxed_path(),
+                    },
+                ))));
             }
 
             if metadata.is_file() {
                 let Some(egg_metadata) = read_metadata(path) else {
                     return Ok(None);
                 };
-                return Ok(Some(Self::EggInfoDirectory(InstalledEggInfoDirectory {
-                    name: file_name.name,
-                    version: Version::from_str(&egg_metadata.version)?,
-                    path: path.to_path_buf().into_boxed_path(),
-                })));
+                return Ok(Some(Self::from(InstalledDistKind::EggInfoDirectory(
+                    InstalledEggInfoDirectory {
+                        name: file_name.name,
+                        version: Version::from_str(&egg_metadata.version)?,
+                        path: path.to_path_buf().into_boxed_path(),
+                    },
+                ))));
             }
         }
 
@@ -268,14 +331,16 @@ impl InstalledDist {
                 return Ok(None);
             };
 
-            return Ok(Some(Self::LegacyEditable(InstalledLegacyEditable {
-                name: egg_metadata.name,
-                version: Version::from_str(&egg_metadata.version)?,
-                egg_link: path.to_path_buf().into_boxed_path(),
-                target: target.into_boxed_path(),
-                target_url: DisplaySafeUrl::from(url),
-                egg_info: egg_info.into_boxed_path(),
-            })));
+            return Ok(Some(Self::from(InstalledDistKind::LegacyEditable(
+                InstalledLegacyEditable {
+                    name: egg_metadata.name,
+                    version: Version::from_str(&egg_metadata.version)?,
+                    egg_link: path.to_path_buf().into_boxed_path(),
+                    target: target.into_boxed_path(),
+                    target_url: DisplaySafeUrl::from(url),
+                    egg_info: egg_info.into_boxed_path(),
+                },
+            ))));
         }
 
         Ok(None)
@@ -283,28 +348,50 @@ impl InstalledDist {
 
     /// Return the [`Path`] at which the distribution is stored on-disk.
     pub fn install_path(&self) -> &Path {
-        match self {
-            Self::Registry(dist) => &dist.path,
-            Self::Url(dist) => &dist.path,
-            Self::EggInfoDirectory(dist) => &dist.path,
-            Self::EggInfoFile(dist) => &dist.path,
-            Self::LegacyEditable(dist) => &dist.egg_info,
+        match &self.kind {
+            InstalledDistKind::Registry(dist) => &dist.path,
+            InstalledDistKind::Url(dist) => &dist.path,
+            InstalledDistKind::EggInfoDirectory(dist) => &dist.path,
+            InstalledDistKind::EggInfoFile(dist) => &dist.path,
+            InstalledDistKind::LegacyEditable(dist) => &dist.egg_info,
         }
     }
 
     /// Return the [`Version`] of the distribution.
     pub fn version(&self) -> &Version {
-        match self {
-            Self::Registry(dist) => &dist.version,
-            Self::Url(dist) => &dist.version,
-            Self::EggInfoDirectory(dist) => &dist.version,
-            Self::EggInfoFile(dist) => &dist.version,
-            Self::LegacyEditable(dist) => &dist.version,
+        match &self.kind {
+            InstalledDistKind::Registry(dist) => &dist.version,
+            InstalledDistKind::Url(dist) => &dist.version,
+            InstalledDistKind::EggInfoDirectory(dist) => &dist.version,
+            InstalledDistKind::EggInfoFile(dist) => &dist.version,
+            InstalledDistKind::LegacyEditable(dist) => &dist.version,
+        }
+    }
+
+    /// Return the [`CacheInfo`] of the distribution, if any.
+    pub fn cache_info(&self) -> Option<&CacheInfo> {
+        match &self.kind {
+            InstalledDistKind::Registry(dist) => dist.cache_info.as_ref(),
+            InstalledDistKind::Url(dist) => dist.cache_info.as_ref(),
+            InstalledDistKind::EggInfoDirectory(..) => None,
+            InstalledDistKind::EggInfoFile(..) => None,
+            InstalledDistKind::LegacyEditable(..) => None,
+        }
+    }
+
+    /// Return the [`BuildInfo`] of the distribution, if any.
+    pub fn build_info(&self) -> Option<&BuildInfo> {
+        match &self.kind {
+            InstalledDistKind::Registry(dist) => dist.build_info.as_ref(),
+            InstalledDistKind::Url(dist) => dist.build_info.as_ref(),
+            InstalledDistKind::EggInfoDirectory(..) => None,
+            InstalledDistKind::EggInfoFile(..) => None,
+            InstalledDistKind::LegacyEditable(..) => None,
         }
     }
 
     /// Read the `direct_url.json` file from a `.dist-info` directory.
-    pub fn direct_url(path: &Path) -> Result<Option<DirectUrl>, InstalledDistError> {
+    pub fn read_direct_url(path: &Path) -> Result<Option<DirectUrl>, InstalledDistError> {
         let path = path.join("direct_url.json");
         let file = match fs_err::File::open(&path) {
             Ok(file) => file,
@@ -317,7 +404,7 @@ impl InstalledDist {
     }
 
     /// Read the `uv_cache.json` file from a `.dist-info` directory.
-    pub fn cache_info(path: &Path) -> Result<Option<CacheInfo>, InstalledDistError> {
+    pub fn read_cache_info(path: &Path) -> Result<Option<CacheInfo>, InstalledDistError> {
         let path = path.join("uv_cache.json");
         let file = match fs_err::File::open(&path) {
             Ok(file) => file,
@@ -329,10 +416,27 @@ impl InstalledDist {
         Ok(Some(cache_info))
     }
 
+    /// Read the `uv_build.json` file from a `.dist-info` directory.
+    pub fn read_build_info(path: &Path) -> Result<Option<BuildInfo>, InstalledDistError> {
+        let path = path.join("uv_build.json");
+        let file = match fs_err::File::open(&path) {
+            Ok(file) => file,
+            Err(err) if err.kind() == std::io::ErrorKind::NotFound => return Ok(None),
+            Err(err) => return Err(err.into()),
+        };
+        let build_info =
+            serde_json::from_reader::<BufReader<fs_err::File>, BuildInfo>(BufReader::new(file))?;
+        Ok(Some(build_info))
+    }
+
     /// Read the `METADATA` file from a `.dist-info` directory.
-    pub fn metadata(&self) -> Result<uv_pypi_types::ResolutionMetadata, InstalledDistError> {
-        match self {
-            Self::Registry(_) | Self::Url(_) => {
+    pub fn read_metadata(&self) -> Result<&uv_pypi_types::ResolutionMetadata, InstalledDistError> {
+        if let Some(metadata) = self.metadata_cache.get() {
+            return Ok(metadata);
+        }
+
+        let metadata = match &self.kind {
+            InstalledDistKind::Registry(_) | InstalledDistKind::Url(_) => {
                 let path = self.install_path().join("METADATA");
                 let contents = fs::read(&path)?;
                 // TODO(zanieb): Update this to use thiserror so we can unpack parse errors downstream
@@ -341,13 +445,19 @@ impl InstalledDist {
                         path: path.clone(),
                         err: Box::new(err),
                     }
-                })
+                })?
             }
-            Self::EggInfoFile(_) | Self::EggInfoDirectory(_) | Self::LegacyEditable(_) => {
-                let path = match self {
-                    Self::EggInfoFile(dist) => Cow::Borrowed(&*dist.path),
-                    Self::EggInfoDirectory(dist) => Cow::Owned(dist.path.join("PKG-INFO")),
-                    Self::LegacyEditable(dist) => Cow::Owned(dist.egg_info.join("PKG-INFO")),
+            InstalledDistKind::EggInfoFile(_)
+            | InstalledDistKind::EggInfoDirectory(_)
+            | InstalledDistKind::LegacyEditable(_) => {
+                let path = match &self.kind {
+                    InstalledDistKind::EggInfoFile(dist) => Cow::Borrowed(&*dist.path),
+                    InstalledDistKind::EggInfoDirectory(dist) => {
+                        Cow::Owned(dist.path.join("PKG-INFO"))
+                    }
+                    InstalledDistKind::LegacyEditable(dist) => {
+                        Cow::Owned(dist.egg_info.join("PKG-INFO"))
+                    }
                     _ => unreachable!(),
                 };
                 let contents = fs::read(path.as_ref())?;
@@ -356,13 +466,16 @@ impl InstalledDist {
                         path: path.to_path_buf(),
                         err: Box::new(err),
                     }
-                })
+                })?
             }
-        }
+        };
+
+        let _ = self.metadata_cache.set(metadata);
+        Ok(self.metadata_cache.get().expect("metadata should be set"))
     }
 
     /// Return the `INSTALLER` of the distribution.
-    pub fn installer(&self) -> Result<Option<String>, InstalledDistError> {
+    pub fn read_installer(&self) -> Result<Option<String>, InstalledDistError> {
         let path = self.install_path().join("INSTALLER");
         match fs::read_to_string(path) {
             Ok(installer) => Ok(Some(installer.trim().to_owned())),
@@ -371,39 +484,71 @@ impl InstalledDist {
         }
     }
 
+    /// Return the supported wheel tags for the distribution from the `WHEEL` file, if available.
+    pub fn read_tags(&self) -> Result<Option<&ExpandedTags>, InstalledDistError> {
+        if let Some(tags) = self.tags_cache.get() {
+            return Ok(tags.as_ref());
+        }
+
+        let path = match &self.kind {
+            InstalledDistKind::Registry(dist) => &dist.path,
+            InstalledDistKind::Url(dist) => &dist.path,
+            InstalledDistKind::EggInfoFile(_) => return Ok(None),
+            InstalledDistKind::EggInfoDirectory(_) => return Ok(None),
+            InstalledDistKind::LegacyEditable(_) => return Ok(None),
+        };
+
+        // Read the `WHEEL` file.
+        let contents = fs_err::read_to_string(path.join("WHEEL"))?;
+        let wheel_file = WheelFile::parse(&contents)?;
+
+        // Parse the tags.
+        let tags = if let Some(tags) = wheel_file.tags() {
+            Some(ExpandedTags::parse(tags.iter().map(String::as_str))?)
+        } else {
+            None
+        };
+
+        let _ = self.tags_cache.set(tags);
+        Ok(self.tags_cache.get().expect("tags should be set").as_ref())
+    }
+
     /// Return true if the distribution is editable.
     pub fn is_editable(&self) -> bool {
         matches!(
-            self,
-            Self::LegacyEditable(_) | Self::Url(InstalledDirectUrlDist { editable: true, .. })
+            &self.kind,
+            InstalledDistKind::LegacyEditable(_)
+                | InstalledDistKind::Url(InstalledDirectUrlDist { editable: true, .. })
         )
     }
 
     /// Return the [`Url`] of the distribution, if it is editable.
     pub fn as_editable(&self) -> Option<&Url> {
-        match self {
-            Self::Registry(_) => None,
-            Self::Url(dist) => dist.editable.then_some(&dist.url),
-            Self::EggInfoFile(_) => None,
-            Self::EggInfoDirectory(_) => None,
-            Self::LegacyEditable(dist) => Some(&dist.target_url),
+        match &self.kind {
+            InstalledDistKind::Registry(_) => None,
+            InstalledDistKind::Url(dist) => dist.editable.then_some(&dist.url),
+            InstalledDistKind::EggInfoFile(_) => None,
+            InstalledDistKind::EggInfoDirectory(_) => None,
+            InstalledDistKind::LegacyEditable(dist) => Some(&dist.target_url),
         }
     }
 
     /// Return true if the distribution refers to a local file or directory.
     pub fn is_local(&self) -> bool {
-        match self {
-            Self::Registry(_) => false,
-            Self::Url(dist) => matches!(&*dist.direct_url, DirectUrl::LocalDirectory { .. }),
-            Self::EggInfoFile(_) => false,
-            Self::EggInfoDirectory(_) => false,
-            Self::LegacyEditable(_) => true,
+        match &self.kind {
+            InstalledDistKind::Registry(_) => false,
+            InstalledDistKind::Url(dist) => {
+                matches!(&*dist.direct_url, DirectUrl::LocalDirectory { .. })
+            }
+            InstalledDistKind::EggInfoFile(_) => false,
+            InstalledDistKind::EggInfoDirectory(_) => false,
+            InstalledDistKind::LegacyEditable(_) => true,
         }
     }
 }
 
 impl DistributionMetadata for InstalledDist {
-    fn version_or_url(&self) -> VersionOrUrlRef {
+    fn version_or_url(&self) -> VersionOrUrlRef<'_> {
         VersionOrUrlRef::Version(self.version())
     }
 }
@@ -440,54 +585,54 @@ impl Name for InstalledLegacyEditable {
 
 impl Name for InstalledDist {
     fn name(&self) -> &PackageName {
-        match self {
-            Self::Registry(dist) => dist.name(),
-            Self::Url(dist) => dist.name(),
-            Self::EggInfoDirectory(dist) => dist.name(),
-            Self::EggInfoFile(dist) => dist.name(),
-            Self::LegacyEditable(dist) => dist.name(),
+        match &self.kind {
+            InstalledDistKind::Registry(dist) => dist.name(),
+            InstalledDistKind::Url(dist) => dist.name(),
+            InstalledDistKind::EggInfoDirectory(dist) => dist.name(),
+            InstalledDistKind::EggInfoFile(dist) => dist.name(),
+            InstalledDistKind::LegacyEditable(dist) => dist.name(),
         }
     }
 }
 
 impl InstalledMetadata for InstalledRegistryDist {
-    fn installed_version(&self) -> InstalledVersion {
+    fn installed_version(&self) -> InstalledVersion<'_> {
         InstalledVersion::Version(&self.version)
     }
 }
 
 impl InstalledMetadata for InstalledDirectUrlDist {
-    fn installed_version(&self) -> InstalledVersion {
+    fn installed_version(&self) -> InstalledVersion<'_> {
         InstalledVersion::Url(&self.url, &self.version)
     }
 }
 
 impl InstalledMetadata for InstalledEggInfoFile {
-    fn installed_version(&self) -> InstalledVersion {
+    fn installed_version(&self) -> InstalledVersion<'_> {
         InstalledVersion::Version(&self.version)
     }
 }
 
 impl InstalledMetadata for InstalledEggInfoDirectory {
-    fn installed_version(&self) -> InstalledVersion {
+    fn installed_version(&self) -> InstalledVersion<'_> {
         InstalledVersion::Version(&self.version)
     }
 }
 
 impl InstalledMetadata for InstalledLegacyEditable {
-    fn installed_version(&self) -> InstalledVersion {
+    fn installed_version(&self) -> InstalledVersion<'_> {
         InstalledVersion::Version(&self.version)
     }
 }
 
 impl InstalledMetadata for InstalledDist {
-    fn installed_version(&self) -> InstalledVersion {
-        match self {
-            Self::Registry(dist) => dist.installed_version(),
-            Self::Url(dist) => dist.installed_version(),
-            Self::EggInfoFile(dist) => dist.installed_version(),
-            Self::EggInfoDirectory(dist) => dist.installed_version(),
-            Self::LegacyEditable(dist) => dist.installed_version(),
+    fn installed_version(&self) -> InstalledVersion<'_> {
+        match &self.kind {
+            InstalledDistKind::Registry(dist) => dist.installed_version(),
+            InstalledDistKind::Url(dist) => dist.installed_version(),
+            InstalledDistKind::EggInfoFile(dist) => dist.installed_version(),
+            InstalledDistKind::EggInfoDirectory(dist) => dist.installed_version(),
+            InstalledDistKind::LegacyEditable(dist) => dist.installed_version(),
         }
     }
 }
