@@ -1,5 +1,5 @@
 use uv_configuration::{Reinstall, Upgrade};
-use uv_pep508::PackageName;
+use uv_normalize::PackageName;
 
 /// Tracks locally installed packages that should not be selected during resolution.
 #[derive(Debug, Default, Clone)]
